import dateutil.parser
import dateutil.tz
from datetime import datetime
from typing import Optional



class StaticUtils:
    @staticmethod
    def parse_date(datetime_string: Optional[str]) -> Optional[datetime]:
        if datetime_string is None:
            return None
        return dateutil.parser.parse(datetime_string)

    @staticmethod
    def replace_tz(datetime: datetime, tz_name: str) -> datetime:
        tz_info = dateutil.tz.gettz(tz_name)
        return datetime.replace(tzinfo=tz_info)

    @staticmethod
    def datetime_to_utc(datetime: datetime) -> datetime:
        return datetime.astimezone(dateutil.tz.UTC)



from datetime import datetime, timedelta
import requests
from typing import Any, Optional


class API:
    def __init__(self, username: str, password: str) -> None:
        self.username: str = username
        self.password: str = password
        self.base_url: str = "https://mit.g4severhome.dk/ESI.API/API"
        self.status_url_part: str = "systemstatus/getState"
        self.command_url_part: str = "Commands/invokeAPI"
        self.panel_id: Optional[int] = None
        self.get_state()

    def get_state(self) -> dict[str, Any]:
        url = f"{self.base_url}/{self.status_url_part}"
        body: dict[str, Any] = {"username": self.username, "password": self.password}
        if self.panel_id is not None:
            body["panel_id"] = self.panel_id
        req = requests.post(url, json=body)
        req.raise_for_status()
        return_value = req.json()
        if return_value["Response"] != 0:
            raise Exception(return_value["ResponseDescription"])
        if self.panel_id is None:
            self.panel_id = return_value["panelInfo"]["PanelId"]
        return return_value

    def arm_alarm(self) -> dict[str, Any]:
        url = f"{self.base_url}/{self.command_url_part}"
        body = {
            "email": self.username,
            "password": self.password,
            "methodToInvoke": "Arm",
            "panelId": self.panel_id,
            "partition": 0,
        }
        req = requests.post(url, json=body)
        return req.json()

    def night_arm_alarm(self) -> dict[str, Any]:
        url = f"{self.base_url}/{self.command_url_part}"
        body = {
            "email": self.username,
            "password": self.password,
            "methodToInvoke": "Arm",
            "partition": 2,
            "panelId": self.panel_id,
        }
        req = requests.post(url, json=body)
        return req.json()

    def day_arm_alarm(self) -> dict[str, Any]:
        url = f"{self.base_url}/{self.command_url_part}"
        body = {
            "email": self.username,
            "password": self.password,
            "methodToInvoke": "Arm",
            "partition": 1,
            "panelId": self.panel_id,
        }
        req = requests.post(url, json=body)
        return req.json()

    def disarm_alarm(self) -> dict[str, Any]:
        url = f"{self.base_url}/{self.command_url_part}"
        body = {
            "email": self.username,
            "password": self.password,
            "methodToInvoke": "Disarm",
            "panelId": self.panel_id,
        }
        req = requests.post(url, json=body)
        return req.json()

    def change_user_panel_pin(self, user_id: int, access_code: str) -> dict[str, Any]:
        url = f"{self.base_url}/users/SetTr5AccessCode"
        body = {
            "panelId": self.panel_id,
            "userName": self.username,
            "password": self.password,
            "accessCode": access_code,
            "userId": user_id,
        }
        req = requests.post(url, json=body)
        req.raise_for_status()
        return req.json()

    def get_events(self, event_type_list: Optional[list[str]]=None, count: int=100, date: Optional[datetime]=None) -> dict[str, Any]:
        url = f"{self.base_url}/Events/InvokeApi"
        body = {
            "email": self.username,
            "password": self.password,
            "methodToInvoke": "GetEventsHistory",
            "panelId": self.panel_id,
            "eventTypeList": "314,56,57,51,58,59,1,2,5,153,155,3,156,8,9,1010,39,40,203,211,212,215,216,222,223,411,412,506,510,511,513,514,515,516,904,906,907,909,910,912,913,915,916,918,919,921,104,106,105,101,103,102,150,152,151,204,1201,1207,1253,706,705,220,221,107,109,112,113,114,418,1258,1259,1260,1875,1876,1205,1208,1602,1601,1604,1610,1611,1640,1641,1612,1613,1614,1615,1616,927,928,930,931,933,901,903,108,810,811,812,813,814,927,815,1617,1618,1810,1811,1891,1892,1893,1894,1895,1896,30,1812,1879,1883,1884,1899,1900,1901,1902,1903",
            "numberOfEvents": count,
        }
        if event_type_list is not None:
            body["eventTypeList"] = ",".join(event_type_list)

        if date is not None:
            body["fromDate"] = date.strftime("%Y-%m-%d")
            body["toDate"] = (date + timedelta(days=1)).strftime("%Y-%m-%d")
        req = requests.post(url, json=body)
        req.raise_for_status()
        return req.json()



from g4s.utils.static_utils import StaticUtils
from datetime import datetime
from typing import Any, Optional


class TimeZone:
    def __init__(self, input_dict: dict[str, Any]) -> None:
        self.country_code: str = input_dict["CountryCode"]
        self.time_zone_id: int = input_dict["TimeZoneId"]
        self.olson_name: str = input_dict["OlsonName"]
        self.is_enabled: bool = input_dict["IsEnabled"]
        self.name: str = input_dict["Name"]

    def date_time_as_utc(self, string_date: Optional[str]) -> Optional[datetime]:
        date_time: Optional[datetime] = StaticUtils.parse_date(string_date)
        if date_time is None:
            return None
        date_time = StaticUtils.replace_tz(date_time, self.olson_name)
        date_time = StaticUtils.datetime_to_utc(date_time)
        return date_time



from datetime import datetime
from g4s.utils.time_zone import TimeZone
from g4s.utils.enums import DeviceType
from typing import Any, Optional


class StateDevice:
    def __init__(self, input_dict: dict[str, Any], timeZone: TimeZone) -> None:
        self.key: Optional[int] = input_dict["key"]
        self.is_tampered: Optional[bool] = input_dict["isTampered"]
        self.has_low_battery: Optional[bool] = input_dict["hasLowBattery"]
        self.has_supervision_fault: Optional[bool] = input_dict["hasSupervisionFault"]
        self.is_open: Optional[bool] = input_dict["isOpen"]
        self.is_locked: bool = input_dict["isLocked"]
        self.is_lockout: Optional[bool] = input_dict["isLockout"]
        self.is_triggered_alarm: bool = input_dict["isTriggeredAlarm"]
        self.alarm_type: int = input_dict["alarmType"]
        self.user_name: Optional[str] = input_dict["userName"]
        self.serial_number: str = input_dict["serialNumber"]
        self.day_partition: bool = input_dict["dayPartition"]
        self.night_partition: bool = input_dict["nightPartition"]
        self.rf_level: Optional[int] = input_dict["rfLevel"]
        self.battery_level: Optional[int] = input_dict["batteryLevel"]
        self.temperature_level: Optional[int] = input_dict["temperatureLevel"]
        self.sub_type: Optional[int] = input_dict["subType"]
        self.attributes: dict[str, Any] = input_dict["attributes"]
        self.hardware_device_type: Optional[int] = input_dict["hardwareDeviceType"]
        self.role_group_id: int = input_dict["roleGroupId"]
        self.bypass_state: int = input_dict["bypassState"]
        self.lock_changed_by_user: Any = input_dict["lockChangedByUser"]
        self.lock_changed_by_device_number: Any = input_dict["lockChangedByDeviceNumber"]
        self.lock_changed_by_device_type: Any = input_dict["lockChangedByDeviceType"]
        self.associated_output_type: Optional[int] = input_dict["associatedOutputType"]
        self.associated_output_id: Any = input_dict["associatedOutputId"]
        self.owner: Optional[int] = input_dict["owner"]
        self.panel_update_time: Optional[datetime] = timeZone.date_time_as_utc(input_dict["panelUpdateTime"])
        self.update_time: Optional[datetime] = timeZone.date_time_as_utc(input_dict["updateTime"])
        self.chime: bool = input_dict["chime"]
        self.security_mode: Optional[int] = input_dict["securityMode"]
        self.is_outdoor_mode: Optional[bool] = input_dict["isOutdoorMode"]
        self.is_beep_enable: Optional[bool] = input_dict["isBeepEnable"]
        self.full_exit_beeps_enabled: Optional[bool] = input_dict["fullExitBeepsEnabled"]
        self.door_bell_enabled: Optional[bool] = input_dict["doorBellEnabled"]
        self.sub_device_type: Any = input_dict["subDeviceType"]
        self.panel_device_id: Any = input_dict["panelDeviceId"]
        self.is_normally_open: Any = input_dict["isNormallyOpen"]
        self.is_pulse_device: Any = input_dict["isPulseDevice"]
        self.ut_device_type: Any = input_dict["utDeviceType"]
        self.additional_data: Any = input_dict["additionalData"]
        self.added_or_reset_time: Optional[datetime] = timeZone.date_time_as_utc(input_dict["addedOrResetTime"])
        self.pk_id: int = input_dict["PkId"]
        self.id: int = input_dict["Id"]
        self.type_id: int = input_dict["Type"]
        self.type: DeviceType = DeviceType(input_dict["Type"])
        self.name: str = input_dict["Name"]
        self.parent_device_id: Any = input_dict["ParentDeviceId"]
        self.panel_id: int = input_dict["PanelId"]
        self.access_code: Optional[str] = input_dict.get("AccessCode")
        self.pet_immune: Optional[bool] = input_dict.get("petImmune")

    def __str__(self) -> str:
        return f"{self.name} - {self.type.name}"

    def __repr__(self) -> str:
        return str(self)



from typing import Any, Optional
from g4s.utils.time_zone import TimeZone
from g4s.utils.state_device import StateDevice


class PanelSettings:
    def __init__(self, input_dict: dict[str, Any]) -> None:
        self.time_zone: TimeZone = TimeZone(input_dict["TimeZone"])
        self.offset_from_utc_in_minutes: int = input_dict["OffsetFromUtcInMinutes"]
        self.temperature_unit: int = input_dict["OffsetFromUtcInMinutes"]
        self.default_temperature_device_id: int = input_dict[
            "DefaultTemperatureDevice"
        ]["Id"]
        self.default_temperature_device: Optional[StateDevice] = None
        self.installed_modules: int = input_dict["InstalledModules"]
        self.primary_link: int = input_dict["PrimaryLink"]
        self.available_delay_times: list[int] = input_dict["AvailableDelayTimes"]
        self.tag_access_code_min_length: int = input_dict["TagAccessCodeMinLength"]
        self.tag_access_code_max_length: int = input_dict["TagAccessCodeMaxLength"]
        self.tag_access_code_available_lengths: list[int] = input_dict["TagAccessCodeAvailableLengths"]
        self.tguard_access_code_length: int = input_dict["GuardAccessCodeLength"]



from datetime import datetime
from typing import Any, Optional
from g4s.utils.time_zone import TimeZone
from g4s.utils.enums import ArmType


class PanelState:
    def __init__(self, input_dict: dict[str, Any], time_zone: TimeZone) -> None:
        self.arm_type: ArmType = ArmType(input_dict["ArmType"])
        self.arm_type_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["ArmTypeChangedTime"]
        )
        self.arm_forced_state: int = input_dict["ArmForcedState"]
        self.arm_delayed_state: int = input_dict["ArmDelayedState"]
        self.alarm_state:int = input_dict["AlarmState"]
        self.alarm_state_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["AlarmStateTime"]
        )
        self.partition: int = input_dict["Partition"]
        self.device_name: Any = input_dict["DeviceName"]
        self.exit_delay_arm_in_process: bool = input_dict["ExitDelayArmInProcess"]
        self.entry_delay_arm_in_process: bool = input_dict["EntryDelayArmInProcess"]
        self.reception_level: int = input_dict["ReceptionLevel"]
        self.reception_level_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["ReceptionLevelChangedTime"]
        )
        self.panel_battery_level: int = input_dict["PanelBatteryLevel"]
        self.is_panel_offline: bool = input_dict["IsPanelOffline"]
        self.is_panel_offline_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["IsPanelOfflineChangedTime"]
        )
        self.is_z_wave_enabled: bool = input_dict["IsZWaveEnabled"]
        self.is_z_wave_enabled_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["IsZWaveEnabledChangedTime"]
        )
        self.is_main_power_connected: bool = input_dict["IsMainPowerConnected"]
        self.is_main_power_connected_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["IsMainPowerConnectedChangedTime"]
        )
        self.is_sim_card_ready: bool = input_dict["IsSimCardReady"]
        self.communication_link: int = input_dict["CommunicationLink"]
        self.backup_channel_status: int = input_dict["BackupChannelStatus"]
        self.backup_channel_status_description: int = input_dict[
            "BackupChannelStatusDescription"]
        
        self.has_low_battery: bool = input_dict["HasLowBattery"]
        self.has_low_battery_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["HasLowBatteryChangedTime"]
        )
        self.setup_mode: bool = input_dict["SetupMode"]
        self.sirens_volume_level: int = input_dict["SirensVolumeLevel"]
        self.sirens_duration: int = input_dict["SirensDuration"]
        self.sirens_volume_level_duration_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["SirensVolumeLevelDurationChangedTime"]
        )
        self.is_in_installation_mode: bool = input_dict["IsInInstallationMode"]
        self.is_in_installation_mode_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["IsInInstallationModeChangedTime"]
        )
        self.is_in_signal_strength_test: bool = input_dict["IsInSignalStrengthTest"]
        self.is_in_signal_strength_test_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["IsInSignalStrengthTestChangedTime"]
        )
        self.panel_id: int = input_dict["PanelId"]
        self.is_synchronized: bool = input_dict["IsSynchronized"]
        self.sirens_entry_exit_duration: int = input_dict["SirensEntryExitDuration"]
        self.frt_state: int = input_dict["FrtState"]
        self.frt_state_changed_time: Optional[datetime] = time_zone.date_time_as_utc(
            input_dict["FrtStateChangedTime"]
        )



from datetime import datetime
from g4s.utils.api import API
from g4s.utils.time_zone import TimeZone
from typing import Any, Optional

class User:
    def __init__(self, input_dict: dict[str, Any], api: API, time_zone: TimeZone) -> None:
        self.api: API = api
        self.id: int = input_dict["id"]
        self.name: str = input_dict["name"]
        self.role_id: int = input_dict["roleId"]
        self.email: str = input_dict["email"]
        self.phone_number: str = input_dict["phoneNumber"]
        self.language_code: int = input_dict["languageCode"]
        self.can_view_comfort_video: bool = input_dict["canViewComfortVideo"]
        self.user_read_terms_and_conditions: bool = input_dict["userReadTermsAndConditions"]
        self.eula_last_updated_time: Optional[datetime] = time_zone.date_time_as_utc(input_dict["eulaLastUpdatedTime"])
        self.user_notifications_settings: dict[str, int] = input_dict["userNotificationsSettings"]
        self.user_storages: dict[str, Any] = input_dict["userStorages"]
        self.password_expiration_days: int = input_dict["passwordExpirationDays"]
        self.can_access_smoke_cannon: bool = input_dict["canAccessSmokeCannon"]
        self.access_code: str = input_dict["accessCode"]
        self.email_confirmation_status: int = input_dict["emailConfirmationStatus"]
        self.package_offerings: list[Any] = input_dict["PackageOfferings"]

    def change_panel_pin(self, new_pin: str):
        self.api.change_user_panel_pin(self.id, new_pin)

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return str(self)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="G4S",  # Replace with your own username
    version="0.1.2",
    author="PTST",
    author_email="patrick@steffensen.io",
    description="API interface for G4S Alarms",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PTST/G4S_Alarm_Py",
    packages=setuptools.find_packages(),
    install_requires=["requests >= 2.23.0", "python-dateutil >= 2.8.1"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from datetime import datetime
from typing import Optional
from g4s.utils.panel_settings import PanelSettings
from g4s.utils.state_device import StateDevice
from g4s.utils.enums import ArmType
from g4s.utils.user import User
import json

from g4s.utils.api import API
from g4s.utils.alarm_status import AlarmStatus


class Alarm:
    def __init__(self, username: str, password: str) -> None:
        self.username: str = username
        self.password: str = password
        self.api: API = API(self.username, self.password)
        self.status: AlarmStatus
        self.users: list[User]
        self.state: ArmType
        self.last_state_change: Optional[datetime]
        self.last_state_change_by: User
        self.sensors: list[StateDevice]
        self.panel_settings: PanelSettings
        self.update_status()

    def update_status(self) -> None:
        self.status = AlarmStatus(self.api.get_state(), self.api)
        self.panel_settings = self.status.panel_settings
        self.users = self.status.users
        self.state = self.status.system_state.arm_type
        self.sensors = self.status.state_devices
        self.last_state_change = self.status.system_state.arm_type_changed_time
        events = json.loads(self.api.get_events(date=self.last_state_change)["Events"])
        matching_events = [
            x
            for x in events
            if self.panel_settings.time_zone.date_time_as_utc(
                x["Events"][0]["Header"]["LocalTime"].replace('"', "")
            )
            == self.last_state_change
            and x["Events"][0]["UserId"] is not None
        ]
        if len(matching_events) > 0:
            user_id = matching_events[0]["Events"][0]["UserId"]
            self.last_state_change_by = [x for x in self.users if x.id == user_id][0]
        self.panel_settings.default_temperature_device = [
            x
            for x in self.sensors
            if x.id == self.panel_settings.default_temperature_device_id
        ][0]

    def arm(self):
        self.api.arm_alarm()
        self.update_status()

    def night_arm(self):
        self.api.night_arm_alarm()
        self.update_status()

    def disarm(self):
        self.api.disarm_alarm()
        self.update_status()

    def __str__(self) -> str:
        return str(self.state)

    def __repr__(self) -> str:
        return str(self)
